@@ -1,5 +1,5 @@
 /*****************************************************************************
- * Secure Locate v1.1                                                        *
+ * Secure Locate v1.2                                                        *
  * October 9, 1998                                                           *
  * Programmed by: Kevin Lindsay                                              *
  * Copyright (c) 1998 Kevin Lindsay & Netnation Communications Inc.          *
@@ -42,19 +42,21 @@
 #include <fcntl.h>
 #include <dirent.h>
 #include <netinet/in.h>
+#include <grp.h>
 
 #include "statmalloc.h"
 #include "link.h"
 
 /* GLOBALS */
 
-#define VERSION "Secure Locate v1.1 - Released October 9, 1998\n"
+#define VERSION "Secure Locate v1.2 - Released October 14, 1998\n"
 #define SLOCATEDB "/var/lib/slocate/slocate.db"
 #define TMPSLOCATEDB "/var/lib/slocate/slocate.tmp"
 #define GRPFILE "/etc/group"
 #define SLOC_ESC 0x80
 #define SLOC_GRP "slocate"
 #define SLOC_UID 0
+#define MIN_BLK 64
 
 char *EXCLUDE_DIR;
 int EXCLUDE=0;
@@ -74,7 +76,7 @@ unsigned short SLOC_GID=65534; /* Seems to be the default nobody gid */
 void
 usage() 
 {
-   printf("Secure Locate v1.1\n"
+   printf("Secure Locate v1.2\n"
           "Copyright (c) 1998 Kevin Lindsay & Netnation Communications Inc.\n"
           "\n"
           "usage: %s [options] <search string>\n\n"
@@ -138,68 +140,34 @@ unsigned short
 get_gid()
 {
    unsigned short GGID=SLOC_GID;
-   FILE *fd;
-   int ch;
-   unsigned int grow=0;
-   char *item;
    int foundit=0;
-   
-   fd = fopen(GRPFILE,"r");
-   
-   if (fd == NULL) {
+	int filefound=0;
+	struct group *grpres;
+	
+   setgrent();
+	while ((grpres = (struct group *)getgrent()) != NULL) {
+		filefound=1;
+		if (!strcmp(grpres->gr_name,SLOC_GRP)) {
+			GGID = grpres->gr_gid;
+			foundit=1;			
+		}
+	}
+	
+	if (filefound)
+		 endgrent();
+	
+	if (!foundit && filefound) {
+      fprintf(stderr,"%s: Could not find the group: %s in the: %s file.\n",progname,SLOC_GRP,GRPFILE);
+      fprintf(stderr,"This is a result of the group missing or a corrupted group file.\n");
+      exit(1);
+   }	
+	
+   if (!filefound) {
       fprintf(stderr,"%s: Could not open %s for reading.\n\n"
               "Read the README.DOC file in the Secure Locate source directory.\n",progname,GRPFILE);
       exit(1);
    }
-   
-   item = (char *)malloc(1);
-   
-   while((ch = getc(fd)) != EOF) {
-      grow++;
-      item = (char *)realloc(item,grow);
-      item[grow-1] = ch;
-      if (ch == ':') {
-         item[grow-1] = '\0';
-
-         if (!strcmp(item,SLOC_GRP)) {
-            
-            while ((ch = getc(fd)) != ':' && ch != EOF);
-            if (ch == EOF) 
-                break;
-            grow=0;
-            free(item);
-            item = (char *)malloc(1);
-            while ((ch = getc(fd)) != ':' && ch != EOF) {
-               grow++;
-               item = (char *)realloc(item,grow);
-               item[grow-1] = ch;
-            }
-            if (ch == EOF)
-                break;
-            
-            grow++;
-            item = (char *)realloc(item,grow);
-            item[grow-1] = '\0';            
-            GGID = atoi(item);
-            free(item);
-            foundit=1;            
-            break;
-         } else {
-            while ((ch = getc(fd)) != '\n' && ch != EOF);
-            if (ch == EOF)
-                break;
-            grow=0;
-         }
-      }
-   }
-   
-   fclose(fd);
-   
-   if (!foundit) {
-      fprintf(stderr,"%s: Could not find the group: %s in the: %s file.\n",progname,SLOC_GRP,GRPFILE);
-      fprintf(stderr,"This is a result of the group missing or a corrupted group file.\n");
-      exit(1);
-   }
+	
    
    return GGID;
 }
@@ -431,14 +399,7 @@ frcode(char *dir_path, char *filename, struct stat statres)
          ptr1++;
          ptr2++;
       }
-/*      printf("I: %d\n",i);
 
-      printf("TMP: %s\n",tmp_path);
-      printf("CUR: %s\n",cur_path);
-      if (i < t_num)
-          fr_num = (strlen(tmp_path) - i) * (-1);
-      else */
-      
       fr_num = i - t_num;
          
       t_num = t_num + fr_num;            
@@ -458,18 +419,17 @@ frcode(char *dir_path, char *filename, struct stat statres)
    if (tmp_path != NULL)
        free(tmp_path);
    
-   
    tmp_path = (char *)malloc(strlen(cur_path)+1);
    tmp_path[0] = '\0';
    strcat(tmp_path,cur_path);
    
    fd = fopen(TMPSLOCATEDB,"a");
-   if (fr_num > -128 && fr_num < 128)
-       putc(fr_num,fd);
-   else if (fr_num > 127 || fr_num < -127) {
+   if (fr_num > 127 || fr_num < -127) {
       putc (SLOC_ESC, fd);
       put_short (fr_num, fd);
-   }
+   } else
+		 putc(fr_num,fd);
+	
    fputs(new_path,fd);
    putc('\0',fd);
 
@@ -662,115 +622,80 @@ decode_db(char *str)
 {
    int res = 1;
    FILE *fd;
-   char *tmppath=NULL;
-   char *codedpath=NULL;
-   char *pcodedpath=NULL;
    short code_num;
-   int codedpathlen=0;
-   char *newpath=NULL;
+	int pathlen=0;
    register char ch;
-   int grow;
+   int grow=0;
    int jump=0;
-   int tmpgrow=0;
+	int first=1;
+	char *codedpath=NULL;
+	char *ptr;
    int printit;
-   unsigned short smode=65535;
-   char *modestr;
-   struct stat statres;
+   struct stat statres;	
   
    if ((fd = fopen(SLOCATEDB,"r")) == NULL) {
       fprintf(stderr,"%s: decode_db(): open: (%d) %s\n",progname,errno,strerror(errno));
       return(0);
    }
-   codedpath = (char *)malloc(1);
 
    slevel = getc(fd);
-   
+	
+	codedpath = (char *)malloc(MIN_BLK);
+	codedpath[0] = '\0';
+   ptr=codedpath;
    while ((code_num = getc(fd)) != EOF) {
+		
       if (code_num == SLOC_ESC) {
-         code_num = get_short(fd);         
-      } else if (code_num > 127 && code_num < 256)
-          code_num = code_num - 256;
-      
-      free(codedpath);
-      codedpath = (char *)malloc(1);
-      grow=0;
+         code_num = get_short(fd);
+      } else if (code_num > 127)
+				 code_num = code_num - 256;
+		
       jump=0;
+		
+		if (code_num < 0)
+			grow += code_num;
+		
+		ptr+=code_num;
+		
+		pathlen = ptr - codedpath;
+
       while(!jump) {
          ch = getc(fd);
          grow++;
-         codedpath = (char *)realloc(codedpath,grow);
-         codedpath[grow-1] = ch;
-         if (ch == '\0') jump=1;
+			pathlen++;
+			if (grow == 64) {
+				codedpath = (char *)realloc(codedpath,pathlen+MIN_BLK);
+				grow=0;
+			}
+         codedpath[pathlen-1] = ch;
+	
+         if (ch == '\0')
+				jump=1;
       }
+		
+		if (first) {
+			ptr=ptr+strlen(codedpath);
+			first=0;
+		}
+		
+		pathlen--;
 
-      codedpathlen=grow-1;
-      
-      if (tmppath == NULL) {
-         
-         newpath = (char *)malloc(codedpathlen+1);
-         newpath[0] = '\0';
-         strcat(newpath,codedpath);
-         
-         pcodedpath = (char *)malloc(codedpathlen+1);
-         pcodedpath[0] = '\0';
-         strcat(pcodedpath,codedpath);
-         
-         tmpgrow+=codedpathlen;         
-         tmppath = (char *)malloc(tmpgrow+1);
-         tmppath[0] = '\0';
-         strcat(tmppath,codedpath);
-         
-      } else {
-
-         tmpgrow+=code_num;
-         if (code_num != 0)
-             tmppath = (char *)realloc(tmppath,tmpgrow+1);
-         if (code_num > 0) {
-            strncat(tmppath,pcodedpath,code_num);
-         } else if (code_num < 0) {
-            tmppath[tmpgrow] = '\0';
-         }
-
-         pcodedpath = (char *)malloc(codedpathlen+1);
-         pcodedpath[0] = '\0';
-         strcat(pcodedpath,codedpath);
-
-         free(newpath);
-         newpath = (char *)malloc(tmpgrow+codedpathlen+1);
-         newpath[0] = '\0';
-         strcat(newpath,tmppath);
-         strcat(newpath,codedpath);
-      }
-      if (strstr(newpath,str) != NULL) {
+      if (strstr(codedpath,str) != NULL) {
          if (slevel == '1') {
-            lstat(newpath,&statres);
-            printit=0;
-            if (UID == 0)
-                printit=1;        
-            else {
-               smode = statres.st_mode & 0777;
-               modestr = (char *)malloc(4);
-               snprintf(modestr,4,"%o\0",smode);
-               
-               if (modestr[2] > '3') {
-                  printit=1;
-               } else if (GID == statres.st_gid && modestr[1] > '3') {
-                  printit=1;
-               } else if (UID == statres.st_uid && modestr[0] > '3') {
-                  printit=1;
-               }
-               
-               free(modestr);
-            }
+				
+            if (UID == 0 || lstat(codedpath,&statres) == 0)
+					 printit=1;
+				else
+					 printit=0;
          } else
              printit=1;
          
          if (printit)
-             printf("%s\n",newpath);
+             printf("%s\n",codedpath);
       }
    }      
    
-   fclose(fd);  
+   fclose(fd);
    
    return(res);
 }
